c
c     Depletion charge/capacitance subroutine, SPICE model
c     for a<=0 and single-piece, smooth model for a>0.
c
      subroutine QCDEPL(qj,cj,vj,p,m,f,a)
      double precision qj,cj,vj,p,m,f,a
      double precision w,xx,vl,vl0,dv,dv0,q0,mv,mv0,dvl,dq
      implicit double precision (a-z)
      if ( a .le. 0.0 ) then
c
c	Standard SPICE depletion charge model
c
	w = 1.0 - vj / p
	if ( w .ge. 1.0 - f ) then
	  cj = w ** ( - m )
	  qj = p * ( 1.0 - w * cj ) / ( 1.0 - m )
	else
	  xx = ( 1.0 - f ) ** ( - ( 1.0 + m ) )
	  cj = xx * ( 1.0 - f * ( 1.0 + m ) + m * vj / p )
	  qj = xx * ( ( 1.0 - f * ( 1.0 + m ) ) * ( vj - f * p ) +
     1		0.5 * m * ( vj * vj - f * f * p * p ) / p ) +
     2		p * ( 1.0 - xx * ( 1.0 - f ) ** 2 ) / ( 1.0 - m )
	endif
      else
c
c	Single-piece, smooth depletion capacitance model
c
	dv0 = - p * f
	mv0 = sqrt ( dv0 * dv0 + a )
	vl0 = 0.5 * ( dv0 - mv0 ) + p * f
	q0  = - p * ( 1.0 - vl0 / p ) ** ( 1.0 - m ) / ( 1.0 - m )
	dv  = vj - p * f
	mv  = sqrt ( dv * dv + a )
	vl  = 0.5 * ( dv - mv ) + p * f
	dq  = ( 1.0 - vl / p ) ** ( - m )
	dvl = 0.5 * ( 1.0 - dv / mv )
	cj  = dq * dvl
	qj  = - p * ( 1.0 - vl / p ) * dq / ( 1.0 - m ) - q0
c
c	Add term to make cj monotonically increasing, to avoid
c	kinks at the seam of depletion and diffusion capacitances
c
	xx  = ( 1.0 - f ) ** ( - m )
	qj  = qj + xx * ( vj  - vl + vl0 )
	cj  = cj + xx * ( 1.0 - dvl )
      endif
      return
      end
c
c     Version for self-heating, that also provides derivative
c     w.r.t. p, which varies with temperature.
c
      subroutine QCDEPLt(qj,cj,dqjdp,vj,p,m,f,a)
      double precision qj,cj,dqjdp,vj,p,m,f,a
      double precision w,xx,vl,vl0,dv,dv0,q0,mv,mv0,dvl,dq
      double precision x1,x2,v2,f2,dvlp0,dvlp
      implicit double precision (a-z)
      if ( a .le. 0.0 ) then
c
c	Standard SPICE depletion charge model
c
	vp = vj / p
	w  = 1.0 - vp
	if ( w .ge. 1.0 - f ) then
	  cj = w ** ( - m )
	  qj = p * ( 1.0 - w * cj ) / ( 1.0 - m )
	  dqjdp = ( qj - cj * vj ) / p
	else
	  xx = ( 1.0 - f ) ** ( - ( 1.0 + m ) )
	  x1 = 1.0 - f * ( 1.0 + m )
	  x2 = ( 1.0 - xx * ( 1.0 - f ) ** 2 ) / ( 1.0 - m )
	  v2 = vp * vp
	  f2 = f * f
	  cj = xx * ( x1 + m * vp )
	  qj = p * x2 + xx * ( x1 * ( vj - f * p ) +
     1		0.5 * m * p * ( v2 - f2 ) )
	  dqjdp = xx * ( - f * x1 - 0.5 * m * ( v2 + f2 ) ) + x2
	endif
      else
c
c	Single-piece, smooth depletion capacitance model
c
	dv0 = - p * f
	mv0 = sqrt ( dv0 * dv0 + a )
	vl0 = 0.5 * ( dv0 - mv0 ) + p * f
	dvlp0 = 0.5 * f * ( 1.0 + dv0 / mv0 )
	dq0 = ( 1.0 - vl0 / p ) ** ( - m )
	q0  = - p * ( 1.0 - vl0 / p ) ** ( 1.0 - m ) / ( 1.0 - m )
	dv  = vj - p * f
	mv  = sqrt ( dv * dv + a )
	vl  = 0.5 * ( dv - mv ) + p * f
	dq  = ( 1.0 - vl / p ) ** ( - m )
	dvl = 0.5 * ( 1.0 - dv / mv )
	dvlp = - f * dvl + f
	cj  = dq * dvl
	qj  = - p * ( 1.0 - vl / p ) * dq / ( 1.0 - m )
c
c	Add term to make cj monotonically increasing, to avoid
c	kinks at the seam of depletion and diffusion capacitances
c
	xx  = ( 1.0 - f ) ** ( - m )
	dqjdp = ( qj - dq * vl ) / p + dq * dvlp
     1		+ xx * ( dvlp0 - dvlp )
     2		- ( ( q0 - dq0 * vl0 ) / p + dq0 * dvlp0 )
	qj  = qj + xx * ( vj  - vl + vl0 ) - q0
	cj  = cj + xx * ( 1.0 - dvl )
      endif
      return
      end
